==> fred-hermes-0.4.0.tar.gz: fred-hermes-0.4.0/hermes/email.py <==
"""High level API for email."""
import logging
from datetime import datetime
from typing import Any, Dict, List, Optional
from uuid import UUID

from fred_api.messenger.common_types_pb2 import MessageStatus
from typist import SecretaryClient
from typist.client import TemplateNotFound

LOGGER = logging.getLogger(__name__)


class Email:
    """Provide a structure for Email message."""

    SECRETARY_CLIENT_DATA_FIELDS = ['subject', 'body', 'body_html']

    def __init__(self,
                 recipient: str,
                 subject_template: str,
                 body_template: str,
                 *,
                 body_template_html: str = None,
                 subject_template_uuid: UUID = None,
                 body_template_uuid: UUID = None,
                 body_template_html_uuid: UUID = None,
                 sender: str = None,
                 extra_headers: Dict[str, str] = None,
                 context: Dict[str, Any] = None,
                 attachments: List[str] = None,
                 type: str = None,
                 secretary_client: SecretaryClient = None):
        """Create Email message."""
        self.sender = sender
        self.recipient = recipient
        self.extra_headers = extra_headers or {}
        self.subject_template = subject_template
        self.body_template = body_template
        self.body_template_html = body_template_html
        self.subject_template_uuid = subject_template_uuid
        self.body_template_uuid = body_template_uuid
        self.body_template_html_uuid = body_template_html_uuid
        self.context = context or {}
        self.attachments = attachments or []
        self.type = type
        self.secretary_client = secretary_client

        self._subject = None  # type: Optional[str]
        self._body = None  # type: Optional[str]
        self._body_html = None  # type: Optional[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> 'Email':
        """Create instance from a dictionary.

        Raises:
            ValueError: If required keys for Email creation are missing

        """
        # recipient and both templates are mandatory
        try:
            email = cls(data['recipient'], data['subject_template'], data['body_template'])
        except KeyError:
            raise ValueError('Required element missing from the data argument.')
        email.extra_headers = data.get('extra_headers', {})
        email.context = data.get('context', {})
        email.attachments = data.get('attachments', [])
        email.sender = data.get('sender')
        email.body_template_html = data.get('body_template_html')
        email.subject_template_uuid = data.get('subject_template_uuid')
        email.body_template_uuid = data.get('body_template_uuid')
        email.body_template_html_uuid = data.get('body_template_html_uuid')
        email.type = data.get('type')
        email.secretary_client = secretary_client
        return email

    @property
    def subject(self):
        """Render subject using django-secretary.

        This property is cached. The template and context attributes are not supposed to be overwritten.
        If you change them, you have to invalidate the cache manually.

        Raises:
            RuntimeError: If secretary client is not set.
        """
        if self._subject:
            return self._subject

        if not self.secretary_client:
            raise RuntimeError("Can't render subject, because secretary_client is not set")

        template = self.subject_template_uuid or self.subject_template
        self._subject = self.secretary_client.render(template, context=self.context)
        return self._subject

    @property
    def body(self):
        """Render body using django-secretary.

        This property is cached. The template and context attributes are not supposed to be overwritten.
        If you change them, you have to invalidate the cache manually.

        Raises:
            RuntimeError: If secretary client is not set.
        """
        if self._body:
            return self._body

        if not self.secretary_client:
            raise RuntimeError("Can't render body, because secretary_client is not set")

        template = self.body_template_uuid or self.body_template
        self._body = self.secretary_client.render(template, context=self.context)
        return self._body

    @property
    def body_html(self):
        """Render html body using django-secretary.

        This property is cached. The template and context attributes are not supposed to be overwritten.
        If you change them, you have to invalidate the cache manually.

        Raises:
            RuntimeError: If secretary client is not set.
        """
        if self._body_html:
            return self._body_html

        template = self.body_template_html_uuid or self.body_template_html
        if not template:
            return None

        if not self.secretary_client:
            raise RuntimeError("Can't render body_html, because secretary_client is not set")

        self._body_html = self.secretary_client.render_html(template, context=self.context)
        return self._body_html

    def _get_secretary_client_data(self) -> Dict[str, Any]:
        """Return dictionary with SecretaryClient field values and render errors.

        When getting SecretaryClient data, which are being rendered, a TemplateNotFound error can occur. In this case
        None is provided as the value and a record is added into the returned dictionary in the ``render_errors`` list.
        """
        ret = {}  # type: Dict[str, Any]
        errors = []

        for item in self.SECRETARY_CLIENT_DATA_FIELDS:
            try:
                ret[item] = getattr(self, item)
            except (RuntimeError, TemplateNotFound, ValueError) as e:
                ret[item] = None
                errors.append('SecretaryClient raised an error in the "{}" property: {}'.format(item, e))
                LOGGER.warning('SecretaryClient raised an error in the "{}" property: {}'.format(item, e))

        if errors:
            ret['render_errors'] = errors

        return ret

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with e-mail info.

        This method is useful for email serialization.
        """
        return {
            'sender': self.sender,
            'recipient': self.recipient,
            'extra_headers': self.extra_headers,
            'subject_template': self.subject_template,
            'body_template': self.body_template,
            'body_template_html': self.body_template_html,
            'subject_template_uuid': self.subject_template_uuid,
            'body_template_uuid': self.body_template_uuid,
            'body_template_html_uuid': self.body_template_html_uuid,
            'context': self.context,
            'attachments': self.attachments,
            **(self._get_secretary_client_data() if self.secretary_client else {}),
        }


class EmailEnvelope:
    """Provide a structure for Email envelope."""

    def __init__(self,
                 message: Email,
                 *,
                 uid: str = None,
                 create_datetime: datetime = None,
                 send_datetime: datetime = None,
                 delivery_datetime: datetime = None,
                 attempts: int = 0,
                 status: MessageStatus = None,
                 archive: bool = True,
                 message_id: str = None):
        self.uid = uid
        self.create_datetime = create_datetime
        self.send_datetime = send_datetime
        self.delivery_datetime = delivery_datetime
        self.attempts = attempts
        self.status = status
        self.message = message
        self.archive = archive
        self.message_id = message_id

    @classmethod
    def from_dict(cls, data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> 'EmailEnvelope':
        """Create instance from a dictionary.

        Raises:
            ValueError: If required keys for Email creation are missing

        """
        email = Email.from_dict(data.get('message_data', {}), secretary_client=secretary_client)
        email_envelope = cls(email)
        email_envelope.uid = data.get('uid')
        email_envelope.create_datetime = data.get('create_datetime')
        email_envelope.send_datetime = data.get('send_datetime')
        email_envelope.delivery_datetime = data.get('delivery_datetime')
        email_envelope.attempts = data.get('attempts', 0)
        email_envelope.status = data.get('status')
        email_envelope.archive = data.get('archive', True)
        email_envelope.message_id = data.get('message_id')
        return email_envelope

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with e-mail envelope info.

        This method is useful for email envelope serialization.
        """
        return {
            'uid': self.uid,
            'create_datetime': self.create_datetime,
            'send_datetime': self.send_datetime,
            'delivery_datetime': self.delivery_datetime,
            'attempts': self.attempts,
            'status': self.status,
            'message': self.message.to_dict(),
            'archive': self.archive,
            'message_id': self.message_id,
        }

==> fred-hermes-0.5.1.tar.gz: fred-hermes-0.5.1/hermes/email.py <==
"""High level API for email."""
from typing import Any, Dict, List, Optional
from uuid import UUID

from typist import SecretaryClient

from .message import Message, MessageEnvelope


class Email(Message):
    """Provide a structure for Email message."""

    rendered_fields = ('subject', 'body', 'body_html')

    def __init__(self,
                 recipient: str,
                 subject_template: str,
                 body_template: str,
                 *,
                 body_template_html: str = None,
                 subject_template_uuid: UUID = None,
                 body_template_uuid: UUID = None,
                 body_template_html_uuid: UUID = None,
                 sender: str = None,
                 extra_headers: Dict[str, str] = None,
                 context: Dict[str, Any] = None,
                 attachments: List[str] = None,
                 type: str = None,
                 secretary_client: SecretaryClient = None):
        """Create Email message."""
        super().__init__(type=type, secretary_client=secretary_client)
        self.sender = sender
        self.recipient = recipient
        self.extra_headers = extra_headers or {}
        self.subject_template = subject_template
        self.body_template = body_template
        self.body_template_html = body_template_html
        self.subject_template_uuid = subject_template_uuid
        self.body_template_uuid = body_template_uuid
        self.body_template_html_uuid = body_template_html_uuid
        self.context = context or {}
        self.attachments = attachments or []

    @property
    def subject(self) -> str:
        """Render subject using django-secretary."""
        return self._secretary_render('subject', template=self.subject_template_uuid or self.subject_template)

    @property
    def body(self) -> str:
        """Render body using django-secretary."""
        return self._secretary_render('body', template=self.body_template_uuid or self.body_template)

    @property
    def body_html(self) -> Optional[str]:
        """Render body_html using django-secretary."""
        template = self.body_template_html_uuid or self.body_template_html
        if template:
            return self._secretary_render(
                'body_html',
                template=template,
                render_method_name='render_html',
            )
        else:
            return None

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with e-mail info.

        This method is useful for email serialization.
        """
        info = super().to_dict()
        info.update({
            'sender': self.sender,
            'recipient': self.recipient,
            'extra_headers': self.extra_headers,
            'subject_template': self.subject_template,
            'body_template': self.body_template,
            'body_template_html': self.body_template_html,
            'subject_template_uuid': self.subject_template_uuid,
            'body_template_uuid': self.body_template_uuid,
            'body_template_html_uuid': self.body_template_html_uuid,
            'context': self.context,
            'attachments': self.attachments,
        })
        return info


class EmailEnvelope(MessageEnvelope[Email]):
    """Provide a structure for Email envelope."""

    message_cls = Email

    def __init__(self, message: Email, *, message_id: str = None, **kwargs):
        super().__init__(message=message, **kwargs)
        self.message_id = message_id

    @classmethod
    def from_dict(cls, data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> 'EmailEnvelope':
        """Create instance from a dictionary.

        Raises:
            ValueError: If required keys for Email creation are missing
        """
        email_envelope = super().from_dict(data, secretary_client=secretary_client)
        email_envelope.message_id = data.get('message_id')
        return email_envelope

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with e-mail envelope info.

        This method is useful for email envelope serialization.
        """
        info = super().to_dict()
        info.update({'message_id': self.message_id})
        return info
